import yaml
import os
import subprocess
import urllib.parse

from charmhelpers import fetch
from charmhelpers.core import (
    hookenv,
    host,
    templating,
)
from charmhelpers.payload import execd

from charms import reactive
from charms.reactive import hook, when
from charms.reactive.decorators import when_file_changed


PACKAGES = ('python-dev', 'python-virtualenv', 'python-yaml', 'python-jinja2', 'libyaml-0-2', 'libyaml-dev')
SERVICE_NAME = 'graphite-beacon'


@hook('install')
def install():
    basenode()
    ensure_directories()
    install_packages(PACKAGES)
    install_python_packages()


def get_base_directory():
    base_dir = '/srv/{}'.format(SERVICE_NAME)
    return base_dir


def ensure_directories():
    base_dir = get_base_directory()
    log_dir = os.path.join(base_dir, 'logs')
    etc_dir = os.path.join(base_dir, 'etc')

    for dirpath in [base_dir, log_dir, etc_dir]:
        host.mkdir(dirpath, owner='root', group='ubuntu')


def basenode():
    hookenv.log("Executing basenode")
    execd.execd_preinstall()


def install_packages(packages):
    hookenv.log('Installing dependencies...')
    fetch.configure_sources(update=True)
    fetch.apt_install(packages, fatal=True)


def install_python_packages(upgrade=False):
    base_dir = get_base_directory()
    env_dir = os.path.join(base_dir, 've')
    if os.path.exists(env_dir) and not upgrade:
        hookenv.log('Service venv already exists')
    elif not os.path.exists(env_dir):
        hookenv.log('Installing python packages into the venv ...')
        subprocess.check_call(['virtualenv', '-p', 'python', env_dir])

    pip_cache = os.path.join(hookenv.charm_dir(), 'files', 'pip-cache')
    args = []
    if upgrade:
        args.append('--upgrade')

    if os.path.exists(pip_cache):
        hookenv.log('Installing from cache.')
        args.extend(['--no-index', '--find-links={}'.format(pip_cache)])
    else:
        hookenv.log("No pip_cache payload found, trying to use pypi")
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install'] + args + [SERVICE_NAME])
    # install pyyaml, for yml config support
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install'] + args + ['PyYAML'])


@hook('upgrade-charm')
def upgrade_charm():
    install_packages(PACKAGES)
    install_python_packages(upgrade=True)


@hook('config-changed')
def config_changed():
    base_dir = get_base_directory()
    etc_dir = os.path.join(base_dir, 'etc')
    config = hookenv.config()

    if config['alerts']:
        alerts_content = config['alerts']
        alerts_path = os.path.join(etc_dir, 'alerts.yml')
        with open(alerts_path, 'w') as f:
            f.write(alerts_content)

    if config['config']:
        config_content = config['config']
        beacon_config = yaml.load(config_content)
        if config['alerts']:
            beacon_config['include'] = [os.path.join(etc_dir, 'alerts.yml')]
        config_path = os.path.join(etc_dir, 'config.yml')
        with open(config_path, 'w') as f:
            f.write(yaml.dump(beacon_config))
        render_upstart_config()
    else:
        hookenv.log("Missing config, not starting service.")


def render_upstart_config():
    templating.render(
        source='upstart.conf',
        target='/etc/init/{}.conf'.format(SERVICE_NAME),
        context=hookenv.config().copy()),
    reactive.set_state('graphite-beacon.ready')


@when_file_changed(os.path.join(get_base_directory(), 'etc', 'config.yml'),
                   os.path.join(get_base_directory(), 'etc', 'alerts.yml'))
def restart():
    if not os.path.exists('/etc/init/{}.conf'.format(SERVICE_NAME)):
        return
    host.service_restart('graphite-beacon')


@when('nrpe-external-master.available')
def setup_nagios(nagios):
    hookenv.log("Updating nrpe configs")
    config = hookenv.config()
    for rel_id in hookenv.relation_ids("nrpe-external-master"):
        hookenv.relation_set(rel_id, primary=True)
    nagios.add_check(
        ["/usr/lib/nagios/plugins/check_procs", "-c", "1:", "-C", "graphite-beacon"],
        context=config['nagios_context'],
        name="graphite-beacon-running",
        description="graphite-beacon process is running",
        unit=hookenv.local_unit())


@when('conn-check.available')
def conn_check_joined(conn_check):
    config = hookenv.config()
    parsed = urllib.parse.urlparse(config['graphite_url'])
    host, port = urllib.parse.splitport(parsed.netloc)
    if not port:
        port = 80
    check = {'host': host,
             'port': port,
             'type': 'tcp'}
    if parsed.scheme == 'https':
        check['type'] = 'tls'
        if not port:
            port = 443
    conn_check.add_check("graphite_url", check)
    conn_check.update(nagios_context=config['nagios_context'])

